@@ -205,7 +205,7 @@ parse_cnf()
 get_footprint()
 {
     pushd $WSREP_SST_OPT_DATA 1>/dev/null
-    payload=$(du --block-size=1 -c  **/*.ibd **/*.MYI **/*.MYI ibdata1  | awk 'END { print $1 }')
+    payload=$(find . -regex '.*\.ibd$\|.*\.MYI$\|.*\.MYD$\|.*ibdata1$' -type f -print0 | xargs -0 du --block-size=1 -c | awk 'END { print $1 }')
     if my_print_defaults -c $WSREP_SST_OPT_CONF xtrabackup | grep -q -- "--compress";then 
         # QuickLZ has around 50% compression ratio
         # When compression/compaction used, the progress is only an approximate.
@@ -383,6 +383,7 @@ wait_for_listen()
 
 check_extra()
 {
+    local use_socket=1
     if [[ $uextra -eq 1 ]];then 
         if my_print_defaults -c $WSREP_SST_OPT_CONF mysqld | tr '_' '-' | grep -- "--thread-handling=" | grep -q 'pool-of-threads';then 
             local eport=$(my_print_defaults -c $WSREP_SST_OPT_CONF mysqld | tr '_' '-' | grep -- "--extra-port=" | cut -d= -f2)
@@ -391,6 +392,7 @@ check_extra()
                 # Hence, setting host to 127.0.0.1 unconditionally. 
                 wsrep_log_info "SST through extra_port $eport"
                 INNOEXTRA+=" --host=127.0.0.1 --port=$eport "
+                use_socket=0
             else 
                 wsrep_log_error "Extra port $eport null, failing"
                 exit 1
@@ -399,6 +401,9 @@ check_extra()
             wsrep_log_info "Thread pool not set, ignore the option use_extra"
         fi
     fi
+    if [[ $use_socket -eq 1 ]] && [[ -n "${WSREP_SST_OPT_SOCKET}" ]];then
+        INNOEXTRA+=" --socket=${WSREP_SST_OPT_SOCKET}"
+    fi
 }
 
 if [[ ! -x `which innobackupex` ]];then 
@@ -698,6 +703,11 @@ then
         wsrep_log_info "${IST_FILE} received from donor: Running IST"
     fi
 
+    if [[ ! -r ${MAGIC_FILE} ]];then
+        wsrep_log_error "SST magic file ${MAGIC_FILE} not found/readable"
+        exit 2
+    fi
+
     cat "${MAGIC_FILE}" # output UUID:seqno
     wsrep_log_info "Total time on joiner: $totime seconds"
 fi
