@@ -26,6 +26,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include "log_event.h"
+#include <slave.h>
 
 Format_description_log_event *wsrep_format_desc = NULL;
 wsrep_t *wsrep                  = NULL;
@@ -59,7 +60,10 @@ ulong   wsrep_mysql_replication_bundle = 0;
 my_bool wsrep_desync                   = 0; // desynchronize the node from the
                                             // cluster
 my_bool wsrep_load_data_splitting      = 1; // commit load data every 10K intervals
-
+my_bool wsrep_restart_slave            = 0; // should mysql slave thread be
+                                            // restarted, if node joins back
+my_bool wsrep_restart_slave_activated  = 0; // node has dropped, and slave
+                                            // restart will be needed
 /*
  * End configuration options
  */
@@ -121,7 +125,7 @@ static void wsrep_log_cb(wsrep_log_level_t level, const char *msg) {
     sql_print_error("WSREP: %s", msg);
     break;
   case WSREP_LOG_DEBUG:
-    if (wsrep_debug) sql_print_information ("[Debug] WSREP: %s", msg);
+    sql_print_information ("[Debug] WSREP: %s", msg);
   default:
     break;
   }
@@ -226,6 +230,10 @@ wsrep_view_handler_cb (void*                    app_ctx,
 
   /* Proceed further only if view is PRIMARY */
   if (WSREP_VIEW_PRIMARY != view->status) {
+#ifdef HAVE_QUERY_CACHE
+    // query cache must be initialised by now
+    query_cache.flush();
+#endif /*HAVE_QUERY_CACHE*/
     wsrep_ready_set(FALSE);
     new_status= WSREP_MEMBER_UNDEFINED;
     /* Always record local_uuid and local_seqno in non-prim since this
@@ -381,14 +389,45 @@ void wsrep_ready_wait ()
 static void wsrep_synced_cb(void* app_ctx)
 {
   WSREP_INFO("Synchronized with group, ready for connections");
+  bool signal_main= false;
   if (mysql_mutex_lock (&LOCK_wsrep_ready)) abort();
   if (!wsrep_ready)
   {
     wsrep_ready= TRUE;
     mysql_cond_signal (&COND_wsrep_ready);
+    signal_main= true;
+
   }
   local_status.set(WSREP_MEMBER_SYNCED);
   mysql_mutex_unlock (&LOCK_wsrep_ready);
+
+  if (signal_main)
+  {
+      wsrep_SE_init_grab();
+      // Signal mysqld init thread to continue
+      wsrep_sst_complete (&local_uuid, local_seqno, false);
+      // and wait for SE initialization
+      wsrep_SE_init_wait();
+  }
+  if (wsrep_restart_slave_activated)
+  {
+    int rcode;
+    WSREP_INFO("MySQL slave restart");
+    wsrep_restart_slave_activated= FALSE;
+
+    mysql_mutex_lock(&LOCK_active_mi);
+    if ((rcode = start_slave_threads(1 /* need mutex */,
+                            0 /* no wait for start*/,
+                            active_mi,
+                            master_info_file,
+                            relay_log_info_file,
+                       	    SLAVE_SQL)))
+    {
+      WSREP_WARN("Failed to create slave threads: %d", rcode);
+    }
+    mysql_mutex_unlock(&LOCK_active_mi);
+
+  }
 }
 
 static void wsrep_init_position()
@@ -609,6 +648,7 @@ int wsrep_init()
   {
     DBUG_PRINT("wsrep",("wsrep::init() failed: %d", rcode));
     WSREP_ERROR("wsrep::init() failed: %d, must shutdown", rcode);
+    wsrep->free(wsrep);
     free(wsrep);
     wsrep = NULL;
   }
@@ -622,7 +662,7 @@ void wsrep_init_startup (bool first)
 {
   if (wsrep_init()) unireg_abort(1);
 
-  wsrep_thr_lock_init(wsrep_thd_is_brute_force, wsrep_abort_thd,
+  wsrep_thr_lock_init(wsrep_thd_is_BF, wsrep_abort_thd,
                       wsrep_debug, wsrep_convert_LOCK_to_trx, wsrep_on);
 
   /* Skip replication start if no cluster address */
@@ -1138,7 +1178,7 @@ create_view_query(THD *thd, uchar** buf, size_t* buf_len)
     }
     buff.append(STRING_WITH_LEN(" AS "));
     //buff.append(views->source.str, views->source.length);
-    buff.append(thd->lex->create_view_select.str, 
+    buff.append(thd->lex->create_view_select.str,
                 thd->lex->create_view_select.length);
     //int errcode= query_error_code(thd, TRUE);
     //if (thd->binlog_query(THD::STMT_QUERY_TYPE,
@@ -1147,7 +1187,7 @@ create_view_query(THD *thd, uchar** buf, size_t* buf_len)
 }
 
 static int wsrep_TOI_begin(THD *thd, char *db_, char *table_,
-                           const TABLE_LIST* table_list) 
+                           const TABLE_LIST* table_list)
 {
   wsrep_status_t ret(WSREP_WARNING);
   uchar* buf(0);
@@ -1172,7 +1212,7 @@ static int wsrep_TOI_begin(THD *thd, char *db_, char *table_,
     buf_err= wsrep_create_event_query(thd, &buf, &buf_len);
     break;
   default:
-    buf_err= wsrep_to_buf_helper(thd, thd->query(), thd->query_length(), &buf, 
+    buf_err= wsrep_to_buf_helper(thd, thd->query(), thd->query_length(), &buf,
                                  &buf_len);
     break;
   }
@@ -1224,7 +1264,7 @@ static void wsrep_TOI_end(THD *thd) {
   }
 }
 
-static int wsrep_RSU_begin(THD *thd, char *db_, char *table_) 
+static int wsrep_RSU_begin(THD *thd, char *db_, char *table_)
 {
   wsrep_status_t ret(WSREP_WARNING);
   WSREP_DEBUG("RSU BEGIN: %lld, %d : %s", (long long)wsrep_thd_trx_seqno(thd),
@@ -1306,9 +1346,9 @@ int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
   int ret= 0;
   mysql_mutex_lock(&thd->LOCK_wsrep_thd);
 
-  if (thd->wsrep_conflict_state == MUST_ABORT) 
+  if (thd->wsrep_conflict_state == MUST_ABORT)
   {
-    WSREP_INFO("thread: %lu, %s has been aborted due to multi-master conflict", 
+    WSREP_INFO("thread: %lu, %s has been aborted due to multi-master conflict",
                thd->thread_id, thd->query());
     mysql_mutex_unlock(&thd->LOCK_wsrep_thd);
     return WSREP_TRX_FAIL;
@@ -1318,9 +1358,16 @@ int wsrep_to_isolation_begin(THD *thd, char *db_, char *table_,
   DBUG_ASSERT(thd->wsrep_exec_mode == LOCAL_STATE);
   DBUG_ASSERT(thd->wsrep_trx_meta.gtid.seqno == WSREP_SEQNO_UNDEFINED);
 
+  if (thd->global_read_lock.can_acquire_protection())
+  {
+    WSREP_DEBUG("Aborting TOI: Global Read-Lock (FTWRL) in place: %s %lu",
+                thd->query(), thd->thread_id);
+    return -1;
+  }
+
   if (wsrep_debug && thd->mdl_context.has_locks())
   {
-    WSREP_DEBUG("thread holds MDL locks at TI begin: %s %lu", 
+    WSREP_DEBUG("thread holds MDL locks at TI begin: %s %lu",
                 thd->query(), thd->thread_id);
   }
 
@@ -1412,7 +1459,7 @@ wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
       mysql_mutex_unlock(&granted_thd->LOCK_wsrep_thd);
       ret = TRUE;
     }
-    else if (request_thd->lex->sql_command == SQLCOM_DROP_TABLE) 
+    else if (request_thd->lex->sql_command == SQLCOM_DROP_TABLE)
     {
       WSREP_DEBUG("DROP caused BF abort");
       ticket->wsrep_report(wsrep_debug);
@@ -1420,7 +1467,7 @@ wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
       wsrep_abort_thd((void*)request_thd, (void*)granted_thd, 1);
       ret = FALSE;
     }
-    else if (granted_thd->wsrep_query_state == QUERY_COMMITTING) 
+    else if (granted_thd->wsrep_query_state == QUERY_COMMITTING)
     {
       WSREP_DEBUG("mdl granted, but commiting thd abort scheduled");
       ticket->wsrep_report(wsrep_debug);
@@ -1428,7 +1475,7 @@ wsrep_grant_mdl_exception(MDL_context *requestor_ctx,
       wsrep_abort_thd((void*)request_thd, (void*)granted_thd, 1);
       ret = FALSE;
     }
-    else 
+    else
     {
       WSREP_MDL_LOG(DEBUG, "MDL conflict-> BF abort", request_thd, granted_thd);
       ticket->wsrep_report(wsrep_debug);
