@@ -50,7 +50,9 @@ extern const char wsrep_defaults_file[];
 #define WSREP_SST_MYSQLDUMP    "mysqldump"
 #define WSREP_SST_RSYNC        "rsync"
 #define WSREP_SST_SKIP         "skip"
-#define WSREP_SST_DEFAULT      WSREP_SST_RSYNC
+#define WSREP_SST_XTRABACKUP      "xtrabackup"
+#define WSREP_SST_XTRABACKUP_V2   "xtrabackup-v2"
+#define WSREP_SST_DEFAULT      WSREP_SST_XTRABACKUP_V2
 #define WSREP_SST_ADDRESS_AUTO "AUTO"
 #define WSREP_SST_AUTH_MASK    "********"
 
@@ -132,7 +134,7 @@ static bool sst_auth_real_set (const char* value)
           {
             my_free ((void*)wsrep_sst_auth);
             wsrep_sst_auth = my_strdup(WSREP_SST_AUTH_MASK, MYF(0));
-            //strncpy (wsrep_sst_auth, WSREP_SST_AUTH_MASK, 
+            //strncpy (wsrep_sst_auth, WSREP_SST_AUTH_MASK,
             //     sizeof(wsrep_sst_auth) - 1);
           }
           else
@@ -227,7 +229,13 @@ void wsrep_sst_complete (const wsrep_uuid_t* sst_uuid,
   }
   else
   {
-    WSREP_WARN("Nobody is waiting for SST.");
+    /* This can happen when called from wsrep_synced_cb().
+       At the moment there is no way to check there
+       if main thread is still waiting for signal,
+       so wsrep_sst_complete() is called from there
+       each time wsrep_ready changes from FALSE -> TRUE.
+    */
+    WSREP_DEBUG("Nobody is waiting for SST.");
   }
   mysql_mutex_unlock (&LOCK_wsrep_sst);
 }
@@ -708,10 +716,11 @@ static int sst_donate_mysqldump (const char*         addr,
               WSREP_SST_OPT_LPORT" '%u' "
               WSREP_SST_OPT_SOCKET" '%s' "
               WSREP_SST_OPT_DATA" '%s' "
+              WSREP_SST_OPT_CONF" '%s' "
               WSREP_SST_OPT_GTID" '%s:%lld'"
               "%s",
               user, pswd, host, port, mysqld_port, mysqld_unix_port,
-              mysql_real_data_home, uuid_str,
+              mysql_real_data_home, wsrep_defaults_file, uuid_str,
               (long long)seqno, bypass ? " "WSREP_SST_OPT_BYPASS : "");
 
     WSREP_DEBUG("Running: '%s'", cmd_str);
@@ -895,8 +904,9 @@ wait_signal:
     else
     {
       WSREP_ERROR("Failed to read from: %s", proc.cmd());
+      proc.wait();
     }
-    if (err && proc.error()) err= proc.error();
+    if (!err && proc.error()) err= proc.error();
   }
   else
   {
@@ -1007,7 +1017,10 @@ void wsrep_SE_init_grab()
 
 void wsrep_SE_init_wait()
 {
-  mysql_cond_wait (&COND_wsrep_sst_init, &LOCK_wsrep_sst_init);
+  while (SE_initialized == false)
+  {
+    mysql_cond_wait (&COND_wsrep_sst_init, &LOCK_wsrep_sst_init);
+  }
   mysql_mutex_unlock (&LOCK_wsrep_sst_init);
 }
 
